EXPERIMENT_PATH = "experiment-path"
EXPERIMENT_LABELS = "experiment-labels"
PROBE_DIAMETER = "probe-diameter"
LOAD_EXPERIMENT = "load-experiment"
LOAD_OUTPUT = "load-output"
EXPERIMENT = "experiment"
IMAGES = "images"

CURVE_DROPDOWN = "curve-dropdown"
CURVE_DATA = "curve-data"
LOAD_ANIMATION = "load-animation"
BUTTON_BACK = "button-back"
BUTTON_FORWARD = "button-forward"

IMAGE_FRAME = "image-frame"
LOADIMAGE_ANIMATION = "loadimage-animation"

UPLOAD_ANNOTATIONS = "upload-annotations"
CP_ANNOTATIONS = "cp-annotations"
VD_ANNOTATIONS = "vd-annotations"

VD_CHECKLIST = "vd-checklist"
ADJUST_CHECKLIST = "adjust-checklist"
LOADCONTACT_ANIMATION = "loadcontact-animation"
DETECT_CONTACT = "detect-contact"
RESET_CONTACT = "reset-contact"
FIT_CHECKLIST = "fit-checklist"
INDENTATION = "indentation"

DOWNLOAD_ANNOTATIONS = "download-annotations"
DOWNLOAD_IMAGEDATA = "download-imagedata"
DOWNLOAD_FITS = "download-fits"
DOWNLOAD_CURVES = "download-curves"
DOWNLOAD_EXPERIMENT = "download-experiment"
DOWNLOAD_ANIMATION = "download-animation"
DOWNLOAD = 'download'

FIG_HOLDER = "fig-holder"
CONTACT_FIG = "contact-fig"
FORCETIME_FIG = "forcetime-fig"



from dash import Dash, dcc
from dash.dependencies import Input, Output, State
from dash.exceptions import PreventUpdate
import os

from ...afm import AFM
from ..components import ids
from ..utils.utils import dump, make_json


def render(app: Dash) -> dcc.Store:
    @app.callback(
        [Output(ids.EXPERIMENT, 'data', allow_duplicate=True),Output(ids.CP_ANNOTATIONS, 'data', allow_duplicate=True),
         Output(ids.VD_ANNOTATIONS, 'data', allow_duplicate=True)],
        [Input(ids.LOAD_EXPERIMENT, 'n_clicks')],
        [State(ids.EXPERIMENT_PATH, 'value'),
         State(ids.EXPERIMENT_LABELS, 'value'),
         State(ids.PROBE_DIAMETER, 'value')],
        prevent_initial_call=True
    )
    def store_experiment_info(_, experiment_path, labels, probe_diameter):
        if not experiment_path or not os.path.exists(experiment_path) or not os.path.isdir(experiment_path):
            raise PreventUpdate

        if not labels or not probe_diameter:
            raise PreventUpdate

        exp_name = os.path.basename(os.path.normpath(experiment_path))
        path = os.path.dirname(os.path.normpath(experiment_path))
        label_list = [label.strip() for label in labels.split(';')]
        experiment = AFM(path, exp_name, label_list, float(probe_diameter))
        experiment.experiment.reduce_data()
        cp_data = make_json(experiment.curve_keys, 0)
        vd_data = make_json(experiment.curve_keys, False)
        return dump(experiment), cp_data, vd_data

    return dcc.Store(id=ids.EXPERIMENT)



from dash import Dash, dcc, html
from dash.dependencies import Input, Output, State


from . import ids


def render(app: Dash) -> html.Div:

    return html.Div(
        children=[
            html.Div(
                children=[
                    dcc.Checklist(
                        id=ids.FIT_CHECKLIST,
                        options=[
                            {'label': '  Show Fits', 'value': True}],
                        style={
                            'width': "25%",
                        },
                    ),
                    dcc.Input(
                        placeholder="Enter max indentation or interval...",
                        id=ids.INDENTATION,
                        style={
                            'width': "75%",
                        },
                    )
                ],
                style={
                    'display': 'flex',
                    'width': '100%',
                }
            ),
            dcc.Loading(
                id=ids.DOWNLOAD_ANIMATION,
                type="dot",
                children=html.Div(
                    children=[
                        html.Button(
                            children="Download Fits",
                            id=ids.DOWNLOAD_FITS,
                            n_clicks=0,
                            className="dash-button"
                        ),
                        html.Button(
                            children="Download Curves",
                            id=ids.DOWNLOAD_CURVES,
                            n_clicks=0,
                            className="dash-button"
                        ),
                        html.Button(
                            children="Download Experiment",
                            id=ids.DOWNLOAD_EXPERIMENT,
                            n_clicks=0,
                            className="dash-button"
                        ),
                    ],
                    style={
                        'display': 'flex',
                        'gap': '5px',
                    },
                )
            )
        ],
        style={
            'display': 'flex',
            'flex-direction': 'column',
            'width': '100%',
            'gap': '5px',
            'align-items': 'start',
        }
    )



from dash import Dash, html, callback_context
from dash.exceptions import PreventUpdate
from dash.dependencies import Input, Output, State

import json
import os
from . import ids, fig
from ..utils.utils import get_current_annotation
from ...asylum import load_ibw


def render(app: Dash) -> html.Div:
    @app.callback(
        [Output(ids.CONTACT_FIG, 'figure', allow_duplicate=True),
         Output(ids.FORCETIME_FIG, 'figure', allow_duplicate=True)],
        [Input(ids.CURVE_DROPDOWN, 'value'),
         Input(ids.ADJUST_CHECKLIST, 'value'),
         Input(ids.VD_ANNOTATIONS, 'data'),
         Input(ids.CP_ANNOTATIONS, 'data')],
        [State(ids.EXPERIMENT_PATH, 'value'),
         State(ids.CONTACT_FIG, 'figure'),
         State(ids.FORCETIME_FIG, 'figure'),
         ],
        prevent_initial_call=True
    )
    def show_data(curve_value, adjust, vd_data, cp_data, experiment_path, contact_fig, forcetime_fig):
        ctx = callback_context
        if not ctx.triggered or not curve_value or not vd_data or not cp_data:
            raise PreventUpdate
        trigger_id = ctx.triggered[0]['prop_id'].split('.')[0]

        vd = get_current_annotation(curve_value, vd_data)
        if trigger_id == ids.CP_ANNOTATIONS and not vd and not adjust:
            raise PreventUpdate
        print('raft too')
        name = eval(curve_value)['name'] + ".ibw"
        file_name = os.path.join(experiment_path, name)
        curve = load_ibw(file_name)
        curve.reduce_data()
        cp = get_current_annotation(curve_value, cp_data)
        curve.set_contact_index(cp)
        curve.get_figs_data(vd=vd, adjust=adjust)
        contact_fig = curve.get_contact_fig_plot()
        forcetime_fig = curve.get_dwell_relax_fig_plot()
        if not trigger_id in (ids.VD_ANNOTATIONS, ids.CP_ANNOTATIONS):
            contact_fig.update_layout(
                xaxis={'autorange': True},
                yaxis={'autorange': True},
            )
            forcetime_fig.update_layout(
                xaxis={'autorange': True},
                yaxis={'autorange': True},
            )

        return contact_fig, forcetime_fig

    @app.callback(
        Output(ids.CP_ANNOTATIONS, 'data'),
        [Input(ids.CONTACT_FIG, 'clickData')],
        [State(ids.CURVE_DROPDOWN, 'value'),
         State(ids.CP_ANNOTATIONS, 'data')],
        prevent_initial_call=True
    )
    def handle_click(clickData, curve_value, cp_data):
        key = eval(curve_value)['key']
        cp_annotations = json.loads(cp_data)
        new_selected_index = clickData['points'][0]['pointIndex']
        cp_annotations[repr(key)] = new_selected_index
        return json.dumps(cp_annotations)

    return html.Div(
        className='figure',
        id=ids.FIG_HOLDER,
        style={
            'display': 'flex',
            'width': '100%',
        },
        children=[
            fig.render(id=ids.CONTACT_FIG,
                       title=r"$\text{Selected Contact Point: }$",
                       xaxis=r"$Indentation \text{ (m)}$"),
            fig.render(id=ids.FORCETIME_FIG,
                       title=r"$\text{Dwell and Relaxation}$",
                       xaxis=r"$Time \text{ (s)}$"),
        ],
    )



from dash import Dash, dcc, html, no_update
from dash.exceptions import PreventUpdate
from dash.dependencies import Input, Output, State

import json

from . import ids
from ..utils.utils import load_json, update_annotations


def render(app: Dash) -> html.Div:
    @app.callback(
        [Output(ids.CP_ANNOTATIONS, 'data', allow_duplicate=True),
         Output(ids.VD_ANNOTATIONS, 'data', allow_duplicate=True),
         Output(ids.UPLOAD_ANNOTATIONS, 'contents')],
        [Input(ids.UPLOAD_ANNOTATIONS, 'contents')],
        [State(ids.CP_ANNOTATIONS, 'data'),
         State(ids.VD_ANNOTATIONS, 'data')],
        prevent_initial_call=True
    )
    def load_annotations(encoded_contents, cp_data, vd_data):
        if not encoded_contents or not cp_data or not vd_data:
            raise PreventUpdate

        content_type, content_string = encoded_contents.split(',')
        loaded_annotations = load_json(content_string)
        first_value = next(iter(loaded_annotations.values()))
        if isinstance(first_value, bool):
            vd_annotations = json.loads(vd_data)
            vd_annotations = update_annotations(
                vd_annotations, loaded_annotations)
            return no_update, json.dumps(vd_annotations), None
        elif isinstance(first_value, int):
            cp_annotations = json.loads(cp_data)
            cp_annotations = update_annotations(
                cp_annotations, loaded_annotations)
            return json.dumps(cp_annotations), no_update, None

    return html.Div(
        children=[
            dcc.Store(id=ids.CP_ANNOTATIONS),
            dcc.Store(id=ids.VD_ANNOTATIONS),
            dcc.Upload(
                className='drag-drop',
                children=html.Div(
                    ['Drag and Drop or ', html.A('Select Annotation File'), ' to Load']),
                id=ids.UPLOAD_ANNOTATIONS,
                multiple=False
            ),
        ],
    )



from dash import Dash
from dash_bootstrap_components.themes import BOOTSTRAP

from .src.components.layout import create_layout


def run() -> None:
    app = Dash(__name__, external_stylesheets=[BOOTSTRAP])
    app.title = "AFM Curve Analysis"
    app.layout = create_layout(app)
    app.run(debug=True)


if __name__ == "__main__":
    run()



from dash import Dash, html
from dash.dependencies import Input, Output

from . import ids
from ..utils.utils import load, load_image


def render(app: Dash) -> html.Div:

    @app.callback(
        Output(ids.IMAGE_FRAME, 'src'),
        [Input(ids.CURVE_DROPDOWN, 'value'),
         Input(ids.IMAGES, 'data')],
        prevent_initial_call=True
    )
    def update_image_frame(curve_value, encoded_images):
        if not encoded_images or not curve_value:
            return ''

        images: dict = load(encoded_images)

        key = eval(curve_value)['key']
        new_key = key[:-1] if len(key) > 1 else key

        # THIS ONLY SHOWS THE FIRST IMAGE IF THERE IS ONE
        if images.get(new_key):
            image_path = images[new_key][0]
            image_src = load_image(image_path)
            return image_src
        else:
            return ''

    return html.Div(
        className='image',
        children=[
            html.Img(
                id=ids.IMAGE_FRAME,
                style={'display': 'flex',
                       'margin': '10px auto',
                       'height': '250px'
                       }
            )
        ],
    )



from dash import Dash, dcc, html
from . import (
    ids,
    loader,
    curve_dropdown,
    image_frame,
    toolbox,
    fig_frame,
)


def create_layout(app: Dash) -> html.Div:
    return html.Div(
        className="app-div",
        children=[
            html.H1(app.title),
            html.Hr(),
            dcc.Loading(
                id=ids.LOAD_ANIMATION,
                # There are different types like 'graph', 'cube', 'circle', 'dot', and 'default'
                type="default",
                children=html.Div(
                    className="file-reader",
                    children=[
                        loader.render(app),
                    ],
                ),
            ),
            html.Hr(),
            html.Div(
                className='dashboard',
                children=[
                    html.Div(
                        children=[
                            curve_dropdown.render(app),
                            dcc.Loading(
                                id=ids.LOADIMAGE_ANIMATION,
                                type="default",
                                children=[image_frame.render(app)]
                            ),
                        ],
                        style={
                            'width': '50%',
                        },
                    ),
                    toolbox.render(app),
                ],
                style={
                    'display': 'flex',
                    'width': '100%',
                },
            ),
            html.Hr(),
            fig_frame.render(app)
        ],
    )



from dash import Dash, dcc, no_update
from dash.dependencies import Input, Output, State
import json

from ..components import ids
from ..utils.utils import load, dump
from .processor import process_experiment, process_indentation, get_pdf


def render(app: Dash) -> dcc.Download:
    @app.callback(
        Output(ids.DOWNLOAD, 'data', allow_duplicate=True),
        [Input(ids.DOWNLOAD_ANNOTATIONS, 'n_clicks')],
        [State(ids.CP_ANNOTATIONS, 'data'),
         State(ids.LOAD_OUTPUT, 'children')],
        prevent_initial_call=True
    )
    def download_cp(_, cp_data, exp_output):
        exp_name = exp_output.split('\'')[1]
        data_dict = json.loads(cp_data)

        json_string = json.dumps(data_dict, indent=4)
        return {'content': json_string, 'filename': f'{exp_name}_cp_annotations.json', 'type': 'text/json'}

    @app.callback(
        Output(ids.DOWNLOAD, 'data'),
        [Input(ids.DOWNLOAD_ANNOTATIONS, 'n_clicks')],
        [State(ids.VD_ANNOTATIONS, 'data'),
         State(ids.LOAD_OUTPUT, 'children')],
        prevent_initial_call=True
    )
    def download_vd(_, vd_data, exp_output):
        exp_name = exp_output.split('\'')[1]
        data_dict = json.loads(vd_data)

        json_string = json.dumps(data_dict, indent=4)
        return {'content': json_string, 'filename': f'{exp_name}_vd_annotations.json', 'type': 'text/json'}

    @app.callback(
        [Output(ids.DOWNLOAD, 'data', allow_duplicate=True),
         Output(ids.EXPERIMENT, 'data', allow_duplicate=True),
         Output(ids.DOWNLOAD_FITS, 'children', allow_duplicate=True),
         Output(ids.INDENTATION, 'value')],
        [Input(ids.DOWNLOAD_FITS, "n_clicks")],
        [State(ids.EXPERIMENT, 'data'),
         State(ids.CP_ANNOTATIONS, 'data'),
         State(ids.VD_ANNOTATIONS, 'data'),
         State(ids.INDENTATION, 'value'),
         State(ids.LOAD_OUTPUT, 'children')],
        prevent_initial_call=True
    )
    def download_fits_csv(_, encoded_experiment, cp_data, vd_data, indentation, exp_output):
        if indentation:
            experiment = load(encoded_experiment)
            indentation = process_indentation(indentation)
            experiment_processed, df = process_experiment(
                experiment, cp_data, vd_data, indentation)
            exp_name = exp_output.split('\'')[1]
            return dcc.send_data_frame(df.to_csv, filename=f"{exp_name}_fits.csv"), dump(experiment_processed), no_update, no_update

        return no_update, no_update, no_update, "Unable to proceed without indentation!"

    @app.callback(
        [Output(ids.DOWNLOAD, 'data', allow_duplicate=True),
         Output(ids.DOWNLOAD_CURVES, 'children')],
        [Input(ids.DOWNLOAD_CURVES, "n_clicks")],
        [State(ids.EXPERIMENT, 'data'),
         State(ids.LOAD_OUTPUT, 'children')],
        prevent_initial_call=True
    )
    def download_curves_pdf(_, encoded_experiment, exp_output):
        experiment_processed = load(encoded_experiment)
        exp_name = exp_output.split('\'')[1]
        pdf_src = get_pdf(experiment_processed)

        return dcc.send_bytes(src=pdf_src.getvalue(), filename=f"{exp_name}_curves.pdf", base64=True), no_update

    return dcc.Download(id=ids.DOWNLOAD)



from dash import Dash, dcc, html, callback_context, no_update
from dash.exceptions import PreventUpdate
from dash.dependencies import Input, Output, State

from typing import Protocol
import json

from . import ids, fig
from ..utils.utils import load, get_current_annotation


class Curve(Protocol):
    def detect_contact(self) -> int:
        ...


class CurveSet(Protocol):
    def __getitem__(self, key: tuple[str]) -> Curve:
        ...


class AFM(Protocol):
    experiment: CurveSet


def render(app: Dash) -> html.Div:

    @app.callback(
        [Output(ids.VD_CHECKLIST, 'value', allow_duplicate=True),
         Output(ids.VD_ANNOTATIONS, 'data')],
        [Input(ids.CURVE_DROPDOWN, 'value'),
         Input(ids.VD_ANNOTATIONS, 'data'),
         Input(ids.VD_CHECKLIST, 'value')],
        prevent_initial_call=True
    )
    def show_vd_checklist(curve_value, vd_data, vd_checklist):
        ctx = callback_context
        if not ctx.triggered or not curve_value or not vd_data:
            raise PreventUpdate
        trigger_id = ctx.triggered[0]['prop_id'].split('.')[0]

        key = eval(curve_value)['key']
        vd_annotations = json.loads(vd_data)
        vd = vd_annotations[repr(key)]
        if trigger_id in (ids.CURVE_DROPDOWN, ids.VD_ANNOTATIONS):
            if vd:
                return [vd], no_update
            else:
                return [], no_update
        elif trigger_id == ids.VD_CHECKLIST:
            if vd_checklist:
                vd_annotations[repr(key)] = True
            else:
                vd_annotations[repr(key)] = False
            return no_update, json.dumps(vd_annotations)

    @app.callback(
        Output(ids.CONTACT_FIG, 'figure', allow_duplicate=True),
        [Input(ids.CP_ANNOTATIONS, 'data')],
        [State(ids.ADJUST_CHECKLIST, 'value'),
         State(ids.CURVE_DROPDOWN, 'value'),
         State(ids.CONTACT_FIG, 'figure'),
         State(ids.VD_CHECKLIST, 'value')],
        prevent_initial_call=True
    )
    def set_contact(cp_data, adjust, curve_value, contact_fig_dict, vd):
        if not curve_value or vd or adjust:
            raise PreventUpdate

        cp = get_current_annotation(curve_value, cp_data)
        contact_fig = fig.update_contact_line(cp, contact_fig_dict)
        return contact_fig

    @app.callback(
        [Output(ids.CP_ANNOTATIONS, 'data', allow_duplicate=True),
         Output(ids.DETECT_CONTACT, 'children')],
        [Input(ids.RESET_CONTACT, 'n_clicks'),
         Input(ids.DETECT_CONTACT, 'n_clicks')],
        [State(ids.CURVE_DROPDOWN, 'value'),
         State(ids.CP_ANNOTATIONS, 'data'),
         State(ids.EXPERIMENT, 'data')],
        prevent_initial_call=True
    )
    def update_contact(reset, detect, curve_value, cp_data, encoded_experiment):
        ctx = callback_context
        if not ctx.triggered:
            raise PreventUpdate
        trigger_id = ctx.triggered[0]['prop_id'].split('.')[0]

        key = eval(curve_value)['key']
        cp_annotations = json.loads(cp_data)
        if trigger_id == ids.RESET_CONTACT:
            cp_annotations[repr(key)] = 0
        elif trigger_id == ids.DETECT_CONTACT:
            afm: AFM = load(encoded_experiment)
            cp_annotations[repr(
                key)] = afm.experiment[key].detect_contact()

        return json.dumps(cp_annotations), no_update

    return html.Div(
        children=[
            dcc.Checklist(id=ids.VD_CHECKLIST,
                          options=[{'label': '  Correct Virtual Deflection', 'value': True}]),
            dcc.Checklist(id=ids.ADJUST_CHECKLIST,
                          options=[{'label': '  Adjust to Contact', 'value': True}]),
            dcc.Loading(
                id=ids.LOADCONTACT_ANIMATION,
                type="dot",
                children=html.Div(
                    children=[
                        html.Button(
                            children="Detect Contact",
                            id=ids.DETECT_CONTACT,
                            n_clicks=0,
                            className="dash-button"
                        ),
                        html.Button(
                            children="Reset Contact",
                            id=ids.RESET_CONTACT,
                            n_clicks=0,
                            className="dash-button"
                        ),
                    ],
                    style={
                        'display': 'flex',
                        'gap': '5px',
                    },
                ),
            ),
            html.Button(
                children="Download Annotations",
                id=ids.DOWNLOAD_ANNOTATIONS,
                n_clicks=0,
                className="dash-button"
            ),
        ],
        style={
            'display': 'flex',
            'flex-direction': 'column',
            'gap': '5px',
            'align-items': 'start'
        }
    )



from dash import Dash, dcc
from dash.dependencies import Input, Output, State

import os
import glob

from ..components import ids
from ..utils.utils import (
    dump,
    extract_keys,
    group_values_by_keys
)


def render(app: Dash) -> dcc.Store:
    @app.callback(
        Output(ids.IMAGES, 'data', allow_duplicate=True),
        [Input(ids.EXPERIMENT, 'data')],
        [State(ids.EXPERIMENT_PATH, 'value'),
         State(ids.EXPERIMENT_LABELS, 'value')],
        prevent_initial_call=True
    )
    def store_images(_, experiment_path, labels):
        label_list = [label.strip() for label in labels.split(';')]

        images = {}
        all_images_dict = {}
        for tif_path in glob.glob(os.path.join(experiment_path, '*.tif')):
            curve_name = os.path.basename(tif_path).split('.')[0]
            curve_key = extract_keys(curve_name, label_list)
            all_images_dict[curve_key] = os.path.join(experiment_path, tif_path)
            images = group_values_by_keys(all_images_dict)

        return dump(images)

    return dcc.Store(id=ids.IMAGES)



from dash import Dash, dcc, html
from dash.dependencies import Input, Output, State

import os

from . import ids
from ..data import (
    experiment_loader,
    image_loader
)


def render(app: Dash) -> html.Div:

    @app.callback(
        Output(ids.LOAD_OUTPUT, 'children'),
        Input(ids.LOAD_EXPERIMENT, 'n_clicks'),
        [State(ids.EXPERIMENT_PATH, 'value'),
         State(ids.EXPERIMENT_LABELS, 'value'),
         State(ids.PROBE_DIAMETER, 'value')],
        prevent_initial_call=True
    )
    def update_output(_, experiment_path, labels, probe_diameter):
        if not experiment_path or not os.path.exists(experiment_path) or not os.path.isdir(experiment_path):
            return html.Div("Invalid directory path.", id=ids.LOAD_OUTPUT)

        if not labels or not probe_diameter:
            return html.Div("Labels or probe diameter cannot be empty.", id=ids.LOAD_OUTPUT)

        exp_name = os.path.basename(os.path.normpath(experiment_path))
        return html.Div(f"Experiment '{exp_name}' loading.", id=ids.LOAD_OUTPUT)

    @app.callback(
        Output(ids.EXPERIMENT_PATH, 'style'),
        Input(ids.EXPERIMENT_PATH, 'value'),
        State(ids.EXPERIMENT_PATH, 'style')
    )
    def shrink_output(value, current_style):
        if not value:
            current_style['direction'] = 'ltr'
            return current_style
        current_style['direction'] = 'rtl'
        return current_style

    return html.Div(
        children=[
            html.H5("Load Experiment"),
            html.Div(
                children=[
                    dcc.Input(
                        id=ids.EXPERIMENT_PATH,
                        type='text',
                        placeholder='Enter experiment folder path...',
                        style={
                            'flex': '1',
                            'margin': '2px'}
                    ),
                    dcc.Input(
                        id=ids.EXPERIMENT_LABELS,
                        type='text',
                        placeholder='Enter labels separated by semicolons...',
                        style={
                            'flex': '1',
                            'margin': '2px'}
                    ),
                    dcc.Input(
                        id=ids.PROBE_DIAMETER,
                        type='text',
                        placeholder='Enter probe diameter...',
                        style={
                            'margin': '2px',
                            'width': '20%'}
                    ),
                    html.Button(
                        children="Load",
                        id=ids.LOAD_EXPERIMENT,
                        n_clicks=0,
                        style={
                            'margin': '2px'
                        },
                        className="dash-button"
                    ),
                ],
                style={
                    'display': 'flex',
                    'width': '100%',
                }
            ),
            experiment_loader.render(app),
            image_loader.render(app),
            html.Div(
                children="Enter experiment info and then click load.",
                id=ids.LOAD_OUTPUT
            )
        ],
        className='experiment-loader',
    )



import copy
import os
from . import asylum


class AFM():
    def __init__(
        self,
            path: str,
            exp_name: str,
            labels: list[str],
            probe_diameter: float,
    ) -> None:

        self.path = path
        self.exp_name = exp_name
        self.labels = labels
        self.probe_diameter = probe_diameter

        self.exp_path = os.path.join(self.path, self.exp_name)
        self.load_experiment()
        self.backup_experiment()

    def load_experiment(self) -> None:
        self.experiment = asylum.load_curveset_ibw(
            self.exp_path, self.labels)
        self.curve_keys = self.experiment.keys()

    def backup_experiment(self) -> None:
        self._experiment = copy.deepcopy(self.experiment)
        self._curve_keys = copy.deepcopy(self._experiment.keys())

    def restore_experiment(self) -> None:
        self.experiment = copy.deepcopy(self._experiment)
        self.curve_keys = copy.deepcopy(self._curve_keys)

    def get_key_by_num(self, num: int, label: int = 1, fill: int = 2) -> list[tuple]:
        num_str = str(num).zfill(fill)

        keys = []
        for key in self.curve_keys:
            key_index = key[label]
            if num_str in key_index:
                keys.append(key)
        return keys

    def get_key_by_name(self, name: str, label: int = 1) -> list[tuple]:
        keys = []
        for key in self.curve_keys:
            key_index = key[label]
            if name in key_index:
                keys.append(key)
        return keys

    def drop_curves_by_num(self, drop_idx: list[int], label: int = 1, fill: int = 2) -> None:
        for idx in drop_idx:
            keys = self.get_key_by_num(idx, label=label, fill=fill)
            for key in keys:
                self.experiment.remove_curve(key)



from dash import Dash, html

from ..data import downloader
from . import (
    ids,
    annotator,
    fitter,
    contact_controls
)


def render(app: Dash) -> html.Div:

    return html.Div(
        className='toolbox',
        children=[
            annotator.render(app),
            html.Div(
                children=[
                    contact_controls.render(app),
                    html.Button(
                        children="Download Image Data",
                        id=ids.DOWNLOAD_IMAGEDATA,
                        n_clicks=0,
                        className="dash-button"
                    ),
                    fitter.render(app),
                ],
                style={
                    'display': 'flex',
                    'flex-direction': 'column',
                    'gap': '5px',
                    'align-items': 'start'
                },
            ),
            downloader.render(app),
        ],
        style={
            'width': '50%',
        },
    )
